import { parse } from 'postcss-values-parser';

/** @typedef {Object.<string,number>} StackShape */

/**
 * @typedef Options
 * @property {Function|string[]|StackShape[]} [list]
 * @property {number} [increment]
 */

/** @type {Options} */
const DEFAULT_OPTIONS = {
  list: [],
  increment: 1
};
/**
 * Get the stack based on the list of items
 * @param {Options} argOptions
 * @returns {StackShape}
 */

function getStack(argOptions) {
  const options = { ...DEFAULT_OPTIONS,
    ...argOptions
  };
  const list = Array.isArray(options.list) ? options.list : options.list();
  const explicitList = [];
  const incrementalList = [];
  const listSet = new Set();
  list.forEach(item => {
    const isObject = typeof item === 'object';
    const name = isObject ? Object.keys(item)[0] : item;

    if (!listSet.has(name)) {
      listSet.add(name);

      if (isObject) {
        explicitList.push({
          name,
          zIndex: item[name]
        });
      } else {
        incrementalList.push({
          name,
          zIndex: null
        });
      }
    }
  });
  const combinedLists = [...incrementalList, ...explicitList];
  return combinedLists.map((item, index) => {
    const value = item.zIndex !== null ? item.zIndex : index * options.increment;
    return {
      [item.name]: value
    };
  }).reduce((collection, item) => ({ ...collection,
    ...item
  }), {});
}

const PLUGIN_NAME = 'postcss-stack';
var index = (options => ({
  postcssPlugin: PLUGIN_NAME,

  Once(root, {
    Declaration
  }) {
    const stacked = getStack(options);
    root.walkDecls(decl => {
      const {
        value
      } = decl;

      if (/(^|[^\w-])(stack?)\(/i.test(value)) {
        const ast = parse(value);
        ast.walkFuncs(node => {
          const stackQuoted = node.nodes.filter(item => item.type === 'quoted')[0];
          const quoteRegExp = new RegExp(stackQuoted.quote, 'g');
          const stack = stackQuoted.value.replace(quoteRegExp, '');

          if (typeof stacked[stack] !== 'undefined') {
            // Create a new declaration with just the found stack value
            const replacement = new Declaration({
              prop: '',
              raws: {
                between: ''
              },
              value: String(stacked[stack])
            }); // Replace `stack(STACK_NAME)` func node with new declaration node

            node.replaceWith(replacement);
          } else {
            throw decl.error('Unknown stack', {
              plugin: PLUGIN_NAME,
              word: stack
            });
          }
        });
        const modifiedValue = ast.toString();

        if (value !== modifiedValue) {
          decl.value = modifiedValue;
        }
      }
    });
  }

}));

export { index as default };
export const postcss = true
